@@ -110,7 +110,7 @@ public class ContextConsoleChangeImpl extends BasicCommandlineOptions implements
             dbres = dbctxext.getRestriction();
             final HashSet<Restriction> ret = ResellerAbstraction.handleAddEditRemoveRestrictions(OXResellerTools.array2HashSet(dbres), addres, removeRes, editRes);
             Restriction[] restrictions = null;
-            if( null != ret ) {
+            if ( null != ret ) {
                 restrictions = ret.toArray(new Restriction[ret.size()]);
             }
             if (null == firstExtensionByName) {
@@ -120,7 +120,7 @@ public class ContextConsoleChangeImpl extends BasicCommandlineOptions implements
                 } else {
                     ctxext = new OXContextExtensionImpl();
                 }
-                if( null != customid ) {
+                if ( null != customid ) {
                     ctxext.setCustomid(customid);
                 }
                 ctx.addExtension(ctxext);
@@ -128,28 +128,28 @@ public class ContextConsoleChangeImpl extends BasicCommandlineOptions implements
                 if (null != restrictions) {
                     firstExtensionByName.setRestriction(restrictions);
                 }
-                if( null != customid ) {
+                if ( null != customid ) {
                     firstExtensionByName.setCustomid(customid);
                 }
             }
-        } catch (final InvalidDataException e) {
+        } catch (InvalidDataException e) {
             throw new OXConsolePluginException(e);
-        } catch (final RemoteException e) {
+        } catch (RemoteException e) {
             throw new OXConsolePluginException(e);
-        } catch (final MalformedURLException e) {
+        } catch (MalformedURLException e) {
             throw new OXConsolePluginException(e);
-        } catch (final InvalidCredentialsException e) {
+        } catch (InvalidCredentialsException e) {
             throw new OXConsolePluginException(e);
-        } catch (final StorageException e) {
+        } catch (StorageException e) {
             throw new OXConsolePluginException(e);
-        } catch (final OXResellerException e) {
+        } catch (OXResellerException e) {
             throw new OXConsolePluginException("A reseller exception occured: " + e.getMessage());
-        } catch (final NotBoundException e) {
+        } catch (NotBoundException e) {
             throw new OXConsolePluginException(e);
-        } catch (final DuplicateExtensionException e) {
+        } catch (DuplicateExtensionException e) {
             // Throw this one, but this should never occur as we check beforehand
             throw new OXConsolePluginException(e);
-        } catch (final NoSuchContextException e) {
+        } catch (NoSuchContextException e) {
             throw new OXConsolePluginException(e);
         }
 
