@@ -49,12 +49,13 @@
 
 package com.openexchange.admin.reseller.rmi.exceptions;
 
+import com.openexchange.admin.rmi.exceptions.AbstractAdminRmiException;
 
 /**
  * @author choeger
  *
  */
-public class OXResellerException extends Exception {
+public class OXResellerException extends AbstractAdminRmiException {
 
     public enum Code {
         /**
@@ -82,9 +83,9 @@ public class OXResellerException extends Exception {
          */
         MAXIMUM_NUMBER_CONTEXT_REACHED("Maximum number of contexts reached: %1$s"),
         /**
-         * Maximum overall number of users reached: %1$s
+         * Maximum overall number of contexts reached: %1$s
          */
-        MAXIMUM_OVERALL_NUMBER_OF_CONTEXT_REACHED("Maximum overall number of users reached: %1$s"),
+        MAXIMUM_OVERALL_NUMBER_OF_CONTEXT_REACHED("Maximum overall number of contexts reached: %1$s"),
         /**
          * Maximum overall number of users reached: %1$s
          */
